import { useState, useEffect } from "react";
import { useContext } from "react";
import { useNavigate } from "react-router-dom";
import { CurrentUserContext } from "../../contexts/CurrentUserContext";
import { getUserByUsername } from "../../utils/UserUtils";
import UsersApiService from "../../services/UsersApiService";
import toast from "react-hot-toast";
import { useGroups } from "../../hooks/GroupHook";

const RegisterPage = () => {
    const { currentUser } = useContext(
        CurrentUserContext
    );

    const navigate = useNavigate();

    const { groups } = useGroups();

    if (currentUser) {
        navigate('/');
    }

    let [inputFields, setInputFields] = useState({
        username: "",
        password: "",
        name: "",
        surname: "",
        confirmPassword: "",
        isTeacher: false,
        groupId: null
    });

    const [errors, setErrors] = useState({});
    const [submitting, setSubmitting] = useState(false);

    const validateValues = (inputValues) => {
        let errors = {};
        if (inputValues.username.length <= 0) {
            errors.username = "Введите имя пользователя";
        }
        if (inputValues.password.length <= 0) {
            errors.password = "Введите пароль";
        }
        if (inputValues.name.length <= 0) {
            errors.name = "Введите имя";
        }
        if (inputValues.surname.length <= 0) {
            errors.surname = "Введите фамилию";
        }
        if (inputValues.password != inputValues.confirmPassword) {
            errors.confirmPassword = "Пароли не совпадают";
        }

        if (!inputValues.isTeacher && inputValues.groupId == null) {
            errors.groupId = "Выберите группу";
        }
        return errors;
    };

    const handleChange = (e) => {
        setInputFields({ ...inputFields, [e.target.name]: e.target.value });
        if (e.target.type == 'checkbox') {
            setInputFields({ ...inputFields, [e.target.name]: e.target.checked });
        }
    };

    const handleSubmit = async (event) => {
        event.preventDefault();
        let errors = validateValues(inputFields);
        setErrors(errors);
        if (Object.keys(errors).length === 0) {
            let user = await getUserByUsername(inputFields.username);
            if (user) {
                setErrors({
                    username: "Логин уже занят"
                })
            }
            else {
                await UsersApiService.create({
                    name: inputFields.name,
                    surname: inputFields.surname,
                    username: inputFields.username,
                    password: inputFields.password,
                    isTeacher: inputFields.isTeacher,
                    groupId: parseInt(inputFields.groupId)
                });
                toast.success('Пользователь зарегистрирован!');
            }
        }
        setSubmitting(true);
    };

    const finishSubmit = async () => {
        navigate('/');
    };

    useEffect(() => {
        if (Object.keys(errors).length === 0 && submitting) {
            finishSubmit();
        }
    });

    return (
        <>
            <div className="d-flex flex-fill align-items-center justify-content-center">
                <form onSubmit={handleSubmit} className="d-flex flex-column justify-content-center">
                    <h5 className="text-center">Регистрация</h5>
                    <div className="mb-3">
                        {errors.form}
                    </div>
                    <div className="mb-3 d-flex">
                        <div className="me-3">
                            <input id="name" className="form-control" placeholder="Имя"
                                type="text"
                                name='name'
                                value={inputFields.name}
                                onChange={handleChange}>
                            </input>
                            {errors.name}
                        </div>
                        <div>
                            <input id="surname" className="form-control" placeholder="Фамилия"
                                type="text"
                                name='surname'
                                value={inputFields.surname}
                                onChange={handleChange}>
                            </input>
                            {errors.surname}
                        </div>
                    </div>

                    <div className="mb-3">
                        <input id="username" className="form-control" placeholder="Имя пользователя"
                            type="username"
                            name='username'
                            value={inputFields.username}
                            onChange={handleChange}>
                        </input>
                        {errors.username}
                    </div>
                    <div className="mb-3">
                        <input id="password" className="form-control" placeholder="Пароль"
                            type="password"
                            name='password'
                            value={inputFields.password}
                            onChange={handleChange}
                        >
                        </input>
                        {errors.password}
                    </div>
                    <div className="mb-3">
                        <input id="confirmPassword" className="form-control" placeholder="Повторите пароль"
                            type="password"
                            name='confirmPassword'
                            value={inputFields.confirmPassword}
                            onChange={handleChange}
                        >
                        </input>
                        {errors.confirmPassword}
                    </div>
                    <div className="mb-3 d-flex align-items-center">
                        <input id="isTeacher" className="me-2"
                            type="checkbox"
                            name='isTeacher'
                            value={inputFields.isTeacher}
                            onChange={handleChange}
                        >
                        </input>
                        <label htmlFor="isTeacher">Я преподаватель</label>
                    </div>
                    {
                        inputFields.isTeacher != true ? <div className="mb-3 d-flex">
                            <select name="groupId" id="groupId" className="me-3" onChange={handleChange}>
                                <option value={null}>Выберите группу</option>
                                {
                                    groups.map((group, index) => <option value={group.id} key={index}>{group.name}</option>)
                                }
                            </select>
                            {errors.groupId}
                        </div> : ''
                    }
                    <button id="btn-register-submit" type="submit" className="btn btn-primary" >Зарегистрироваться</button>
                </ form>
            </div>
        </>
    );
};

export default RegisterPage;
